import 'package:freezed_annotation/freezed_annotation.dart';
import 'package:sghi_core/afya_moja_core/afya_moja_core.dart';
import 'package:sghi_core/communities/core/enums.dart';

part 'message.freezed.dart';
part 'message.g.dart';

@freezed
class Message with _$Message {
  factory Message({
    @JsonKey(name: 'type') String? eventType,
    @JsonKey(name: 'sender') String? sender,
    @JsonKey(name: 'content') MessageContent? content,
    @JsonKey(name: 'origin_server_ts') int? timeStamp,
    @JsonKey(name: 'event_id') String? eventID,
    @JsonKey(name: 'room_id') String? roomID,
  }) = _Message;

  factory Message.fromJson(Map<String, dynamic> json) =>
      _$MessageFromJson(json);

  factory Message.initial() => Message(
        eventType: UNKNOWN,
        sender: UNKNOWN,
        content: MessageContent.initial(),
        timeStamp: 0,
        eventID: UNKNOWN,
        roomID: UNKNOWN,
      );
}

@freezed
class MessageContent with _$MessageContent {
  factory MessageContent({
    @JsonKey(name: 'msgtype') String? msgType,
    @JsonKey(name: 'body') String? body,

    /// review these
    @JsonKey(name: 'topic') String? topic,
    @JsonKey(name: 'name') String? name,
    @JsonKey(name: 'guest_access') String? guestAccess,
    @JsonKey(name: 'history_visibility') String? historyVisibility,
    @JsonKey(name: 'join_rule') String? joinRule,
    @JsonKey(name: 'alias') String? alias,
    @JsonKey(name: 'creator') String? creator,
    @JsonKey(name: 'room_version') String? roomVersion,
    @JsonKey(name: 'users') Map<String, dynamic>? users,

    /// For invites and joins
    @JsonKey(name: 'membership') MembershipEventTypes? membership,
    @JsonKey(name: 'displayname') String? displayName,

    // For Images
    @JsonKey(name: 'url') String? url,
  }) = _MessageContent;

  factory MessageContent.fromJson(Map<String, dynamic> json) =>
      _$MessageContentFromJson(json);

  factory MessageContent.initial() => MessageContent(
        msgType: UNKNOWN,
        body: UNKNOWN,

        ///
        topic: UNKNOWN,
        name: UNKNOWN,
        guestAccess: UNKNOWN,
        historyVisibility: UNKNOWN,
        joinRule: UNKNOWN,
        alias: UNKNOWN,
        creator: UNKNOWN,
        roomVersion: UNKNOWN,
        users: <String, dynamic>{},
        membership: MembershipEventTypes.unknown,
        displayName: UNKNOWN,
        url: UNKNOWN,
      );
}
